#!/bin/bash

perl -Ilib t/test.t 1
perl -Ilib t/test.t 2
perl -Ilib t/test.t 3
perl -Ilib t/test.t 4
perl -Ilib t/test.t 5
perl -Ilib t/test.t 6
perl -Ilib t/test.t 7
perl -Ilib t/test.t 8
perl -Ilib t/test.t 9
perl -Ilib t/test.t 10
perl -Ilib t/test.t 11
perl -Ilib t/test.t 12
perl -Ilib t/test.t 13
perl -Ilib t/test.t 14
perl -Ilib t/test.t 15
perl -Ilib t/test.t 16
perl -Ilib t/test.t 17
perl -Ilib t/test.t 18
perl -Ilib t/test.t 19
perl -Ilib t/test.t 20
perl -Ilib t/test.t 21
perl -Ilib t/test.t 22
perl -Ilib t/test.t 23
perl -Ilib t/test.t 24
perl -Ilib t/test.t 25
perl -Ilib t/test.t 26
perl -Ilib t/test.t 27
perl -Ilib t/test.t 28
perl -Ilib t/test.t 29
perl -Ilib t/test.t 30
perl -Ilib t/test.t 31
perl -Ilib t/test.t 32
perl -Ilib t/test.t 33
perl -Ilib t/test.t 34
perl -Ilib t/test.t 35
perl -Ilib t/test.t 36
perl -Ilib t/test.t 37
perl -Ilib t/test.t 38
perl -Ilib t/test.t 39
perl -Ilib t/test.t 40
perl -Ilib t/test.t 41
perl -Ilib t/test.t 42
perl -Ilib t/test.t 43
perl -Ilib t/test.t 44
perl -Ilib t/test.t 45
perl -Ilib t/test.t 46
perl -Ilib t/test.t 47
perl -Ilib t/test.t 48
perl -Ilib t/test.t 49
perl -Ilib t/test.t 50
perl -Ilib t/test.t 51
perl -Ilib t/test.t 52
